//-------------------------------------------------------------------//
//
//	Copyright (c) 2019 BeeBeans Technologies All rights reserved
//
//	System	: 
//
//	Module	: REG_CTL
//
//	Description	: Register Control
//
//	file	: REG_CTL.v
//
//	Note	:
//
//	history	:
//		190422	---------	Created by K.sato

//-------------------------------------------------------------------//

`default_nettype none

module
	REG_CTL	#(
		parameter	[31:0]		VERSION		= 32'h0000_0000,
		parameter	[8*12:0]	FPGA_NAME	= "SOY2_D1"
	)(
		// RBCP2SiTCP Interconnect
		input	wire			SYSCLK_IN,				// System Clock(200MHz)
		input	wire			RESET_IN,				// Register Reset(SiTCP_RST)
		input	wire			RBCP_ACT_IN,			// RBCP active
		input	wire	[31:0]	RBCP_ADDR_IN,			// Address[31:0]
		input	wire	[ 7:0]	RBCP_WD_IN,				// Data[7:0]
		input	wire			RBCP_WE_IN,				// Write enable
		input	wire			RBCP_RE_IN,				// Read enable
		output	reg				RBCP_ACK_OUT,			// Access acknowledge
		output	reg		[ 7:0]	RBCP_RD_OUT,			// Read data[7:0]
		output	reg				BOOT_REQ_OUT,			// Boot Request
		
		// Registers
		//output	reg		[ 7:0]	REG_D1_LED,			// SOY2_D1_LED[3:0]
		input	wire	[ 7:0]	SOY2_D1_COMP_IN,		// SOY2_D1 Comparator[3:0]
		//output	reg		[ 7:0]	REG_D1_Pmod,		// SOY2_D1 Pmod pin[7:0]
		output	reg		[11:0]	REG_D1_DAC_A,			// SOY2_D1 DAC_A[11:0]
		output	reg		[11:0]	REG_D1_DAC_B,			// SOY2_D1 DAC_B[11:0]
		output	reg		[11:0]	REG_D1_DAC_C,			// SOY2_D1 DAC_C[11:0]
		output	reg		[11:0]	REG_D1_DAC_D,			// SOY2_D1 DAC_D[11:0]
		input	wire	[11:0]	SOY2_D1_ADC_A_IN,		// SOY2_D1_ADC_A[11:0]
		input	wire	[11:0]	SOY2_D1_ADC_B_IN,		// SOY2_D1_ADC_B[11:0]
		
		// Registers' Handshaking
		output	reg		[11:0]	P1_WEN,					// RBCP wrote valid[11:0]
		output	reg		[ 7:0]	REG_WDT				// RBCP wrote data[7:0]
//		output	reg		[1:0]	REG_DAC_CH
	);

//
// buffer I/O
//
	reg		[ 5:0]	REG_RAD;
//	reg		[ 7:0]	REG_WDT;
	reg		[ 2:0]	P0_SEL;
	reg				P0_REN;
	reg				P0_WAK;
	reg		[ 3:0]	P0_WEN;
	reg				P1_WAK;
//	reg		[11:0]	P1_WEN;

	wire	[ 7:0]	REG00_DATA	= VERSION[31:24];
	wire	[ 7:0]	REG01_DATA	= VERSION[23:16];
	wire	[ 7:0]	REG02_DATA	= VERSION[15: 8];
	wire	[ 7:0]	REG03_DATA	= VERSION[ 7: 0];

	wire	[ 7:0]	REG04_DATA	= FPGA_NAME[8*12-1:8*11];
	wire	[ 7:0]	REG05_DATA	= FPGA_NAME[8*11-1:8*10];
	wire	[ 7:0]	REG06_DATA	= FPGA_NAME[8*10-1:8* 9];
	wire	[ 7:0]	REG07_DATA	= FPGA_NAME[8* 9-1:8* 8];
	wire	[ 7:0]	REG08_DATA	= FPGA_NAME[8* 8-1:8* 7];
	wire	[ 7:0]	REG09_DATA	= FPGA_NAME[8* 7-1:8* 6];
	wire	[ 7:0]	REG0A_DATA	= FPGA_NAME[8* 6-1:8* 5];
	wire	[ 7:0]	REG0B_DATA	= FPGA_NAME[8* 5-1:8* 4];
	wire	[ 7:0]	REG0C_DATA	= FPGA_NAME[8* 4-1:8* 3];
	wire	[ 7:0]	REG0D_DATA	= FPGA_NAME[8* 3-1:8* 2];
	wire	[ 7:0]	REG0E_DATA	= FPGA_NAME[8* 2-1:8* 1];
	wire	[ 7:0]	REG0F_DATA	= FPGA_NAME[8* 1-1:8* 0];

	wire	[ 7:0]	REG10_DATA;
	wire	[ 7:0]	REG11_DATA;
	wire	[ 7:0]	REG12_DATA;
	wire	[ 7:0]	REG13_DATA;

	wire	[ 7:0]	REG14_DATA;
	wire	[ 7:0]	REG15_DATA;
	wire	[ 7:0]	REG16_DATA;
	wire	[ 7:0]	REG17_DATA;

	wire	[ 7:0]	REG18_DATA;
	wire	[ 7:0]	REG19_DATA;
	wire	[ 7:0]	REG1A_DATA;
	wire	[ 7:0]	REG1B_DATA;

	wire	[ 7:0]	REG1C_DATA;
	wire	[ 7:0]	REG1D_DATA;
	wire	[ 7:0]	REG1E_DATA;
	wire	[ 7:0]	REG1F_DATA;

	reg				P1_RVL;
	reg		[ 7:0]	P10_RDT;
	reg		[ 7:0]	P11_RDT;
	reg		[ 7:0]	P12_RDT;
	reg		[ 7:0]	P13_RDT;
	reg		[ 7:0]	P14_RDT;
	reg		[ 7:0]	P15_RDT;
	reg		[ 7:0]	P16_RDT;
	reg		[ 7:0]	P17_RDT;
	reg				P2_RVL;
	reg		[ 7:0]	P20_RDT;
	reg		[ 7:0]	P21_RDT;
	reg				P3_RVL;
	reg		[ 7:0]	P30_RDT;

	reg				REG_BOOT_REQ;
	

	always@(posedge SYSCLK_IN) begin
		REG_RAD[5:0]	<= (RBCP_RE_IN|RBCP_WE_IN)?		RBCP_ADDR_IN[5:0]:		REG_RAD[5:0];
		REG_WDT[7:0]	<= RBCP_WE_IN?	RBCP_WD_IN[7:0]:	REG_WDT[7:0];
		P0_SEL[0]		<= (RBCP_ADDR_IN[31:19] == 13'b0000_0000_0000_0);
		P0_SEL[1]		<= (RBCP_ADDR_IN[18: 6] == 13'b000_0000_0000_00);
		P0_SEL[2]		<= (RBCP_ADDR_IN[31:24] == 8'h01);
		P0_REN			<= RBCP_RE_IN;
		P0_WAK			<= RBCP_WE_IN;
		P0_WEN[0]		<= RBCP_WE_IN & (RBCP_ADDR_IN[5:2] == 4'b01_00);	// Address 10 - 13
		P0_WEN[1]		<= RBCP_WE_IN & (RBCP_ADDR_IN[5:2] == 4'b01_01);	// Address 14 - 17
		P0_WEN[2]		<= RBCP_WE_IN & (RBCP_ADDR_IN[5:2] == 4'b01_10);	// Address 18 - 1B
		P0_WEN[3]		<= RBCP_WE_IN & (RBCP_ADDR_IN[5:2] == 4'b01_11);	// Address 1C - 1F
		P1_WAK			<= P0_WAK    & (&P0_SEL[1:0]);
		P1_WEN[0]		<= P0_WEN[0] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b00);	// Address 10
		P1_WEN[1]		<= P0_WEN[0] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b01);	// Address 11
		P1_WEN[2]		<= P0_WEN[0] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b10);	// Address 12
		P1_WEN[3]		<= P0_WEN[0] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b11);	// Address 13
		P1_WEN[4]		<= P0_WEN[1] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b00);	// Address 14
		P1_WEN[5]		<= P0_WEN[1] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b01);	// Address 15
		P1_WEN[6]		<= P0_WEN[1] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b10);	// Address 16
		P1_WEN[7]		<= P0_WEN[1] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b11);	// Address 17
		P1_WEN[8]		<= P0_WEN[2] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b00);	// Address 18
		P1_WEN[9]		<= P0_WEN[2] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b01);	// Address 19
		P1_WEN[10]		<= P0_WEN[2] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b10);	// Address 1A
		P1_WEN[11]		<= P0_WEN[2] & (&P0_SEL[1:0]) & (REG_RAD[1:0] == 2'b11);	// Address 1B
	end

	always@(posedge SYSCLK_IN) begin
		P1_RVL			<= P0_REN & (&P0_SEL[1:0]);
		P10_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG00_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG01_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG02_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG03_DATA[7:0]:	8'h00)
		);
		P11_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG04_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG05_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG06_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG07_DATA[7:0]:	8'h00)
		);
		P12_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG08_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG09_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG0A_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG0B_DATA[7:0]:	8'h00)
		);
		P13_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG0C_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG0D_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG0E_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG0F_DATA[7:0]:	8'h00)
		);
		P14_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG10_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG11_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG12_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG13_DATA[7:0]:	8'h00)
		);
		P15_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG14_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG15_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG16_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG17_DATA[7:0]:	8'h00)
		);
		P16_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG18_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG19_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG1A_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG1B_DATA[7:0]:	8'h00)
		);
		P17_RDT[7:0]	<= (
			((REG_RAD[1:0] == 2'b00)?	REG1C_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b01)?	REG1D_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b10)?	REG1E_DATA[7:0]:	8'h00)|
			((REG_RAD[1:0] == 2'b11)?	REG1F_DATA[7:0]:	8'h00)
		);
		P2_RVL		<= P1_RVL;
		P20_RDT[7:0]	<= (
			((REG_RAD[3:2] == 2'b00)?	P10_RDT[7:0]:	8'h00)|
			((REG_RAD[3:2] == 2'b01)?	P11_RDT[7:0]:	8'h00)|
			((REG_RAD[3:2] == 2'b10)?	P12_RDT[7:0]:	8'h00)|
			((REG_RAD[3:2] == 2'b11)?	P13_RDT[7:0]:	8'h00)
		);
		P21_RDT[7:0]	<= (
			((REG_RAD[3:2] == 2'b00)?	P14_RDT[7:0]:	8'h00)|
			((REG_RAD[3:2] == 2'b01)?	P15_RDT[7:0]:	8'h00)|
			((REG_RAD[3:2] == 2'b10)?	P16_RDT[7:0]:	8'h00)|
			((REG_RAD[3:2] == 2'b11)?	P17_RDT[7:0]:	8'h00)
		);

		P3_RVL		<= P2_RVL;
		P30_RDT[7:0]	<= (
			((REG_RAD[5:4] == 2'b00)?	P20_RDT[7:0]:	8'h00)|
			((REG_RAD[5:4] == 2'b01)?	P21_RDT[7:0]:	8'h00)
		);

		RBCP_RD_OUT[7:0]	<= P30_RDT[7:0];
		RBCP_ACK_OUT		<= P3_RVL|P1_WAK;
	end

//	Address : 0000_0010 Control
	assign	REG10_DATA[7:0]	= {7'b0000000,REG_BOOT_REQ};

	always@(posedge SYSCLK_IN or posedge RESET_IN)begin
		if (RESET_IN == 1) begin
			REG_BOOT_REQ	<= 1'b0;
		end else begin
			REG_BOOT_REQ	<= P1_WEN[0]?	REG_WDT[0]:		REG_BOOT_REQ;
		end
	end
	always@(posedge SYSCLK_IN) begin
		BOOT_REQ_OUT	<= REG_BOOT_REQ;
	end

//	Address : 0000_0011 SOY2_D1 LED Control
	assign	REG11_DATA[7:0]	= 8'd0;	//Reservation
	//	assign	REG11_DATA[7:0]	= REG_D1_LED[7:0];
	//	always@(posedge SYSCLK_IN or posedge RESET_IN)begin
	//		if (RESET_IN == 1) begin
	//			REG_D1_LED[7:0]		<= 8'b0000_0000;
	//		end else begin
	//			REG_D1_LED[7:0]	<= P1_WEN[1]?	{4'b0000,REG_WDT[3:0]}:		REG_D1_LED[7:0];
	//		end
	//	end

//	Address : 0000_0012 SOY2_D1 Comparator
	assign	REG12_DATA[7:0]	= {4'b0000,SOY2_D1_COMP_IN[3:0]};

//	Address : 0000_0013 SOY2_D1 Pmod pin
	assign	REG13_DATA[7:0]	= 8'd0;	//Reservation
	//	assign	REG13_DATA[7:0]	= REG_D1_Pmod[7:0];
	//	always@(posedge SYSCLK_IN or posedge RESET_IN)begin
	//		if (RESET_IN == 1) begin
	//			REG_D1_Pmod[7:0]		<= 8'b0000_0000;
	//		end else begin
	//			REG_D1_Pmod[7:0]	<= P1_WEN[3]?	REG_WDT[7:0]:		REG_D1_Pmod[7:0];
	//		end
	//	end

//	Address : 0000_0014-0000_001B SOY2_D1 DAC
	assign	REG14_DATA[7:0]	= {4'b0000,REG_D1_DAC_A[11:8]};
	assign	REG15_DATA[7:0]	= REG_D1_DAC_A[7:0];
	assign	REG16_DATA[7:0]	= {4'b0000,REG_D1_DAC_B[11:8]};
	assign	REG17_DATA[7:0]	= REG_D1_DAC_B[7:0];
	assign	REG18_DATA[7:0]	= {4'b0000,REG_D1_DAC_C[11:8]};
	assign	REG19_DATA[7:0]	= REG_D1_DAC_C[7:0];
	assign	REG1A_DATA[7:0]	= {4'b0000,REG_D1_DAC_D[11:8]};
	assign	REG1B_DATA[7:0]	= REG_D1_DAC_D[7:0];
	
	always@(posedge SYSCLK_IN or posedge RESET_IN)begin
		if (RESET_IN == 1) begin
			REG_D1_DAC_A[11:0]		<= 12'b0;
			REG_D1_DAC_B[11:0]		<= 12'b0;
			REG_D1_DAC_C[11:0]		<= 12'b0;
			REG_D1_DAC_D[11:0]		<= 12'b0;
		end else begin
			REG_D1_DAC_A[11:8]		<= P1_WEN[4]?	REG_WDT[3:0]:	REG_D1_DAC_A[11:8];
			REG_D1_DAC_A[ 7:0]		<= P1_WEN[5]?	REG_WDT[7:0]:	REG_D1_DAC_A[ 7:0];
			REG_D1_DAC_B[11:8]		<= P1_WEN[6]?	REG_WDT[3:0]:	REG_D1_DAC_B[11:8];
			REG_D1_DAC_B[ 7:0]		<= P1_WEN[7]?	REG_WDT[7:0]:	REG_D1_DAC_B[ 7:0];
			REG_D1_DAC_C[11:8]		<= P1_WEN[8]?	REG_WDT[3:0]:	REG_D1_DAC_C[11:8];
			REG_D1_DAC_C[ 7:0]		<= P1_WEN[9]?	REG_WDT[7:0]:	REG_D1_DAC_C[ 7:0];
			REG_D1_DAC_D[11:8]		<= P1_WEN[10]?	REG_WDT[3:0]:	REG_D1_DAC_D[11:8];
			REG_D1_DAC_D[ 7:0]		<= P1_WEN[11]?	REG_WDT[7:0]:	REG_D1_DAC_D[ 7:0];
		end
	end
	
	//	Address : 0000_001C-0000_001F SOY2_D1 ADC
	assign	REG1C_DATA[7:0]	=	{4'b0000,SOY2_D1_ADC_A_IN[11:8]};
	assign	REG1D_DATA[7:0]	=	SOY2_D1_ADC_A_IN[7:0];
	assign	REG1E_DATA[7:0]	=	{4'b0000,SOY2_D1_ADC_B_IN[11:8]};
	assign	REG1F_DATA[7:0]	=	SOY2_D1_ADC_B_IN[7:0];

endmodule

`default_nettype wire
