//----------------------------------------------------------------------//
//
//	Copyright (c) 2019 BeeBeans Technologies All rights reserved
//
//		System		: SOY2_D1
//
//		Module		: SOY2_D1
//
//		Description	: Top module of SOY2_D1
//
//		file		: SOY2_D1.v
//
//		Note		: 
//
//		history	:
//			190419			Created by K.Sato
//
//----------------------------------------------------------------------//

`default_nettype none
`define	FPGA_VER	32'h190419_10
`define	FPGA_NAME	"SOY2_D1"


module SOY2_D1 (
	//System Clock
	input	wire			OSC_IN				,
	// GMII
	output	wire			ETH_nRST_OUT		, // PHY nReset
	output	wire			ETH_PHY_CLK_OUT		, // PHY Clock(25MHz)
	input	wire			ETH_RX_CLK_IN		, // GMII:Rx Clock
	input	wire			ETH_RX_ER_IN		, // GMII:Rx Error
	input	wire			ETH_RX_DV_IN		, // GMII:Rx DataValid
	input	wire	[ 7:0]	ETH_RXD_IN			, // GMII:Rx Data[7:0]
	output	wire			ETH_GTXCLK_OUT		, // GMII:GTX Clock
	input	wire			ETH_TX_CLK_IN		, // GMII:Tx Clock
	output	wire			ETH_TX_ER_OUT		, // GMII:Tx Error
	output	wire			ETH_TX_EN_OUT		, // GMII:Tx Enable
	output	wire	[ 7:0]	ETH_TXD_OUT			, // GMII:Tx Data[7:0]
	input	wire			ETH_COL_IN			, // GMII:Rx Collision detect
	input	wire			ETH_CRS_IN			, // GMII:Rx Carrier Sense
	output	wire			ETH_MDC_OUT			, // PHY MDI Clock
	inout	wire			ETH_MDIO_IO			, // PHY MDI Data
	input	wire			ETH_10_LED_IN		, // GMII:Speed is 10M select LED
	input	wire			ETH_100_LED_IN		, // GMII:Speed is 100M select LED
	input	wire			ETH_1000_LED_IN		, // GMII:Speed is 1000M select LED
	input	wire			ETH_ACT_LED_IN		, // GMII:Active LED
	output	wire			ETH_LED_Y_OUT		, // GMII:FullDuplex LED
	output	wire			ETH_LED_G_OUT		, // GMII:Link LED
	// EEPROM(SiTCP)
	output	wire			EEPROM_CS_OUT		,
	output	wire			EEPROM_SK_OUT		,
	output	wire			EEPROM_DI_OUT		,
	input	wire			EEPROM_DO_IN		,
	// SW
	input	wire			nFD_IN				,	// Jumper2*1
	input	wire			PUSHSW_IN			,	// push switch
	output	wire			PNLLED_G_OUT		,	// 
	output	wire			PNLLED_Y_OUT		,	// 
	output	wire			PNLLED_R_OUT		,	// 
	input	wire	[ 3:0]	DIPSW_IN			,	// Jumper2*4
	// SOY2_D1
	output	wire	[3:0]	SOY2_D1_LED_OUT		,	// out	: SOY2_D1 LED
	input	wire	[3:0]	SOY2_D1_COMP_IN		,	// in	: SOY2_D1 Comparator
	output	wire	[7:0]	SOY2_D1_Pmod_OUT	,	// out	: SOY2_D1 Pmod pin
	output	wire			IIC_SCL_OUT			,
	inout	wire			IIC_SDA_IO			,
	output	reg				IIC_nLDAC			,
	output	reg				IIC_RDY				,
		// SPI INTERFACE
	output wire 		SPI_CLK,				//
	output wire		SPI_CSn,				//	CHIP SELECT (active low)
	output wire		SPI_MOSI,				//
	input  wire		SPI_MISO_A,				//
	input  wire		SPI_MISO_B				//
	
	//input  wire [11:0]	DAC_WDT
	);


	wire			CLK50M;
	wire			OSC50MHz;
	wire			SYSCLK200M;
	wire			CLK_125M;
	wire			PLL_CLKFB;
	wire			SYS_PLL_LOCK;
	reg				SYS_RST;
	reg		[  8:0]	SYS_RST_CNT;
	wire			GMII_MDIO_IN;
	wire			GMII_MDIO_OE;
	wire			GMII_MDIO_OUT;
	reg				GMII_1000M;
	reg		[ 8:0]	CNT_CLK;
	reg				CNT_RST;
	reg				CNT_LD;
	reg		[ 6:0]	RX_CNT;
	wire			ETH_RX_CLK_int;
	wire			IB_ETH_ACT_LED;
	wire			IB_ETH_1000_LED;
	wire			IB_ETH_100_LED;
	wire			IB_ETH_10_LED;
	wire			ETH_LED_Y_int;
	wire			ETH_LED_G_int;
	wire			ETH_RX_ER_int;
	wire			ETH_RX_DV_int;
	wire			GMII_COL;
	wire			GMII_CRS;
	wire			GMII_RX_CLK;
	wire			GMII_RX_DV;
	wire			GMII_RX_ER;
	wire			GMII_TX_CLK;
	wire			GMII_TX_EN;
	wire			GMII_TX_ER;
	wire			IB_TX_CLK;
	wire			GMII_RSTn;
	wire	[ 7:0]	GMII_TXD;
	wire	[ 7:0]	IB_ETH_RXD;
	wire	[ 7:0]	GMII_RXD;
	wire			GMII_MDC;
	wire			SiTCP_RST;
	wire			EEPROM_CS;
	wire			EEPROM_SK;
	wire			EEPROM_DI;
	wire			EEPROM_DO;
	wire			ETH_GTXCLK_int;

	reg		[ 1:0]	CNT25M;
	reg				RE_CLK25M;
	reg				FE_CLK25M;

	wire			ETH_PHY_CLK;

	wire			FORCE_DEFAULTn;
	wire			PUSHSW;
	wire			PNLLED_G;
	wire			PNLLED_Y;
	wire			PNLLED_R;
	wire	[ 3:0]	DIPSW;
	wire			CLK_100M;
	reg		[ 8:0]	SWTIME_1US;
	reg		[ 7:0]	SWTIME_100US;
	reg		[10:0]	SWTIME_100MS;
	reg				P0_SWPLS_100MS;
	reg				P1_SWPLS_100MS;
	reg		[7:0]	CNTLED;

	reg		[7:0]	DEMO_DATA;
	wire			DEMO_ENB;
	reg				reg_SiTCP_TX_WR;
	reg		[7:0]	reg_SiTCP_TX_DATA;
	reg				reg_SiTCP_CLOSE_ACK;
	wire			wi_SiTCP_TX_WR;
	wire	[7:0]	wi_SiTCP_TX_DATA;
	wire			wi_SiTCP_CLOSE_ACK;

	wire	[15:0]	SiTCP_RX_FILL;
	wire	[ 7:0]	SiTCP_RX_DATA;
	wire	[15:0]	SiTCP_TX_FILL;

	wire			SiTCP_OPEN_ACK;
	wire			SiTCP_CLOSE_REQ;
	wire			SiTCP_RX_WR;
	wire			SiTCP_TX_AFULL;
	wire			SiTCP_TX_WR;
	wire	[ 7:0]	SiTCP_TX_DATA;
	wire			SiTCP_CLOSE_ACK;
	wire			TIM_1US;
	wire			TIM_1MS;
	wire			TIM_1S;
	wire			TIM_1MIN;

	wire			BUF_RX_CLK;
	wire	[ 7:0]	IB_RXD;
	wire	[ 7:0]	RXD_DLY;
	wire			IB_RX_ER;
	wire			RX_ER_DLY;
	wire			IB_RX_DV;
	wire			RX_DV_DLY;
	wire			ETH_RX_CLK_DLY;
	wire			ETH_RX_CLK_1DLY;
	wire			ETH_RX_CLK_2DLY;

	reg		[ 1:0]	DEC_ADDR;
	reg		[ 7:0]	P0_RDT;
	reg				P0_RE;
	reg				P0_WE;
//	wire			SIO_LOC_ACK;
//	wire	[ 7:0]	SIO_LOC_RD;
	reg		[ 7:0]	RBCP_DEMODATA;
	reg				RBCP_DEMOACK;
	wire	[31:0]	VERSION_REG	= `FPGA_VER;

//	assign	USR_SUSPEND	= 1'b0;

	wire			REBOOT_REQ;
	wire			BOOT_REQ;
	//wire	[3:0]	REG_D1_LED;
	wire	[3:0]	SOY2_D1_LED;
	wire	[3:0]	SOY2_D1_COMP;
	reg		[7:0]	SOY2_D1_Pmod;
	wire			RBCP_ACT;
	wire	[31:0]	RBCP_ADDR;
	wire	[ 7:0]	RBCP_WD;
	wire			RBCP_WE;
	wire			RBCP_RE;
	wire			RBCP_ACK;
	wire	[ 7:0]	RBCP_RD;

	wire	[11:0]	P1_WEN;
	wire			IIC_WE;
	wire	[ 7:0]	REG_WDT;

//------------------------------------------------------------------------------
// PLL
//------------------------------------------------------------------------------
	IBUFG	OSC_BUF(.O(OSC50MHz),.I(OSC_IN));
//	BUFG	SCK50M_BUF	(.O(OSC50MHz),	.I(CLK50M));

	PLLE2_BASE #(
		.CLKFBOUT_MULT			(20),
		.CLKIN1_PERIOD			(20.000),
		.CLKOUT0_DIVIDE			(5),
		.CLKOUT0_DUTY_CYCLE		(0.500),
		.CLKOUT1_DIVIDE			(8),
		.CLKOUT1_DUTY_CYCLE		(0.500),
		.CLKOUT2_DIVIDE			(10),
		.CLKOUT2_DUTY_CYCLE		(0.500),
		.DIVCLK_DIVIDE			(1)
	) 
	PLLE2_BASE(
		.CLKFBOUT				(PLL_CLKFB),
		.CLKOUT0				(SYSCLK200M),
		.CLKOUT1				(CLK_125M),
		.CLKOUT2				(CLK_100M),
		.CLKOUT3				(),
		.CLKOUT4				(),
		.CLKOUT5				(),
		.LOCKED					(SYS_PLL_LOCK),
		.CLKFBIN				(PLL_CLKFB),
		.CLKIN1					(OSC50MHz),
		.PWRDWN					(1'b0),
		.RST					(1'b0)
	);


//------------------------------------------------------------------------------
// System Reset
//------------------------------------------------------------------------------
	always@ (posedge SYSCLK200M or negedge SYS_PLL_LOCK) begin
		if(~SYS_PLL_LOCK) begin
			SYS_RST_CNT[8:0]    <=    {1'b1,8'd198};
			SYS_RST            <=    1'b1;
		end    else begin
			SYS_RST_CNT[8:0]    <= SYS_RST_CNT[8]?        (SYS_RST_CNT[8:0] - 9'd1):        {1'b0,8'hff};
			SYS_RST            <= SYS_RST_CNT[8];
		end
	end

//------------------------------------------------------------------------------
// GMII/MII Selecter
//------------------------------------------------------------------------------
	always@(posedge SYSCLK200M or posedge SYS_RST)begin
		if (SYS_RST) begin
			CNT_CLK[8:0]	<=	9'b0;
			CNT_LD			<=	1'b0;
			CNT_RST			<=	1'b1;
			GMII_1000M		<=	1'b0;
		end else begin 
			CNT_CLK[8:0]	<=	CNT_CLK[8] ? 9'd198 : CNT_CLK[8:0] - 9'd1;
			CNT_LD			<=	CNT_CLK[8];
			CNT_RST			<=	CNT_LD;
			GMII_1000M		<=	CNT_LD ? RX_CNT[6] : GMII_1000M;
		end
	end
	
	always@(posedge GMII_RX_CLK or posedge CNT_RST)begin
		if (CNT_RST) begin
			RX_CNT[6:0]		<=	7'd0;
		end else begin
			RX_CNT[6:0]		<=	RX_CNT[6] ? RX_CNT[6:0] : RX_CNT[6:0] + 7'd1;
		end
	end

//------------------------------------------------------------------------------
// I/O Buffer
//------------------------------------------------------------------------------

	IBUFG								ETH_TX_CLK_IB	(.O(IB_TX_CLK),.I(ETH_TX_CLK_IN));
	OBUF	#(.DRIVE(4), .SLEW("FAST"))	ETH_TX_EN_OB	(.O(ETH_TX_EN_OUT),	.I(GMII_TX_EN));
	OBUF	#(.DRIVE(4), .SLEW("FAST"))	ETH_TX_ER_OB	(.O(ETH_TX_ER_OUT),	.I(GMII_TX_ER));

	BUFGMUX 							GMIIMUX(.O(GMII_TX_CLK), .I0(IB_TX_CLK), .I1(CLK_125M), .S(GMII_1000M));
	ODDR								IOB_GTX		(.C(GMII_TX_CLK), .CE(1'b1), .D1(1'b0), .D2(1'b1), .R(1'b0), .S(1'b0), .Q(ETH_GTXCLK_int));
	OBUF	#(.DRIVE(4), .SLEW("FAST"))	ETH_GTXCLK_OB	(.O(ETH_GTXCLK_OUT), .I(ETH_GTXCLK_int));

	generate
	genvar ETH_TXD_var ; 
		 for (ETH_TXD_var=0;ETH_TXD_var<8;ETH_TXD_var=ETH_TXD_var+1) begin : ETH_TXD_LOOP
			OBUF	#(.DRIVE(4), .SLEW("FAST"))	ETH_TXD_OB	(.O(ETH_TXD_OUT[ETH_TXD_var]),.I(GMII_TXD[ETH_TXD_var]));
		 end
	endgenerate


	IBUFG								ETH_RX_CLK_IB	(.O(ETH_RX_CLK_int),.I(ETH_RX_CLK_IN));
	BUFR								ETH_RX_CLK_THD	(.O(GMII_RX_CLK), .I(ETH_RX_CLK_int));			// #180926

	generate
		genvar ETH_RXD_var ;
		 for (ETH_RXD_var=0; ETH_RXD_var<8; ETH_RXD_var=ETH_RXD_var+1) begin : ETH_RXD_LOOP
			IBUF						ETH_RXD_IB	(.O(IB_RXD[ETH_RXD_var]),	.I(ETH_RXD_IN[ETH_RXD_var]));
			IDELAYE2 #(
				.IDELAY_TYPE			("FIXED")	,
				.DELAY_SRC				("IDATAIN")	,
				.IDELAY_VALUE			(0)		,
				.HIGH_PERFORMANCE_MODE	("FALSE")	,
				.SIGNAL_PATTERN			("DATA")	,
				.REFCLK_FREQUENCY		(200.0)		,
				.CINVCTRL_SEL			("FALSE")	,
				.PIPE_SEL				("FALSE")
			)
			RXD_IDLY2 (
				.C						(1'b0)		,
				.REGRST					(1'b0)		,
				.LD						(1'b0)		,
				.CE						(1'b0)		,
				.INC					(1'b0)		,
				.CINVCTRL				(1'b0)		,
				.CNTVALUEIN				(5'd0)		,
				.IDATAIN				(IB_RXD[ETH_RXD_var]),
				.DATAIN					(1'b0)		,
				.LDPIPEEN				(1'b0)		,
				.DATAOUT				(GMII_RXD[ETH_RXD_var]),
				.CNTVALUEOUT			()
			);
		 end
	endgenerate


	IBUF						ETH_RX_ER_IB	(.O(IB_RX_ER),	.I(ETH_RX_ER_IN));
	IDELAYE2 #(
		.IDELAY_TYPE			("FIXED")	,
		.DELAY_SRC				("IDATAIN")	,
		.IDELAY_VALUE			(0)		,
		.HIGH_PERFORMANCE_MODE	("FALSE")	,
		.SIGNAL_PATTERN			("DATA")	,
		.REFCLK_FREQUENCY		(200.0)		,
		.CINVCTRL_SEL			("FALSE")	,
		.PIPE_SEL				("FALSE")
	)
	RX_ER_IDLY2 (
		.C						(1'b0)		,
		.REGRST					(1'b0)		,
		.LD						(1'b0)		,
		.CE						(1'b0)		,
		.INC					(1'b0)		,
		.CINVCTRL				(1'b0)		,
		.CNTVALUEIN				(5'd0)		,
		.IDATAIN				(IB_RX_ER)	,
		.DATAIN					(1'b0)		,
		.LDPIPEEN				(1'b0)		,
		.DATAOUT				(GMII_RX_ER)	,
		.CNTVALUEOUT			()
	);


	IBUF						ETH_RX_DV_IB	(.O(IB_RX_DV),	.I(ETH_RX_DV_IN));
	IDELAYE2 #(
		.IDELAY_TYPE			("FIXED")	,
		.DELAY_SRC				("IDATAIN")	,
		.IDELAY_VALUE			(0)		,
		.HIGH_PERFORMANCE_MODE	("FALSE")	,
		.SIGNAL_PATTERN			("DATA")	,
		.REFCLK_FREQUENCY		(200.0)		,
		.CINVCTRL_SEL			("FALSE")	,
		.PIPE_SEL				("FALSE")
	)
	RX_DV_IDLY2 (
		.C						(1'b0)		,
		.REGRST					(1'b0)		,
		.LD						(1'b0)		,
		.CE						(1'b0)		,
		.INC					(1'b0)		,
		.CINVCTRL				(1'b0)		,
		.CNTVALUEIN				(5'd0)		,
		.IDATAIN				(IB_RX_DV)	,
		.DATAIN					(1'b0)		,
		.LDPIPEEN				(1'b0)		,
		.DATAOUT				(GMII_RX_DV),
		.CNTVALUEOUT			()
	);


	IDELAYCTRL	SRAM_DLAYCTL(.RDY(), .REFCLK(SYSCLK200M), .RST(SYS_RST));


	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	ETH_nRST_OUT_OB	(.O(ETH_nRST_OUT),	.I(GMII_RSTn));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	ETH_MDC_OB		(.O(ETH_MDC_OUT),	.I(GMII_MDC));
	IOBUF	#(.DRIVE(4), .SLEW("SLOW"))	ETH_MDIO_BUF	(.O(GMII_MDIO_IN), .IO(ETH_MDIO_IO), .I(GMII_MDIO_OUT), .T(~GMII_MDIO_OE));
	IBUF								ETH_COL_IB		(.O(GMII_COL),		.I(ETH_COL_IN));
	IBUF								ETH_CRS_IB		(.O(GMII_CRS),		.I(ETH_CRS_IN));
	IBUF								ETH_ACT_LED_IB	(.O(IB_ETH_ACT_LED),.I(ETH_ACT_LED_IN));
	IBUF								ETH_10_LED_IB	(.O(IB_ETH_10_LED),	.I(ETH_10_LED_IN));
	IBUF								ETH_100_LED_IB	(.O(IB_ETH_100_LED),.I(ETH_100_LED_IN));
	IBUF								ETH_1000_LED_IB	(.O(IB_ETH_1000_LED),.I(ETH_1000_LED_IN));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	ETH_LED_Y_OB	(.O(ETH_LED_Y_OUT),	.I(ETH_LED_Y_int));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	ETH_LED_G_OB	(.O(ETH_LED_G_OUT),	.I(ETH_LED_G_int));

	assign	ETH_LED_G_int	=	IB_ETH_1000_LED | IB_ETH_100_LED;
    assign  ETH_LED_Y_int   =    IB_ETH_1000_LED | IB_ETH_10_LED;	

	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	EEP_CS_OB		(.O(EEPROM_CS_OUT),	.I(EEPROM_CS));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	EEP_SK_OB		(.O(EEPROM_SK_OUT),	.I(EEPROM_SK));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	EEP_DI_OB		(.O(EEPROM_DI_OUT),	.I(EEPROM_DI));
	IBUF								EEP_DO_IB		(.O(EEPROM_DO),		.I(EEPROM_DO_IN));
	PULLDOWN							EEP_DO_PD		(.O(EEPROM_DO_IN));
/*
	IBUF								USR_CLK_IB		(.O(USR_CLK),		.I(USR_CLK_IN));
	IBUF								USR_CLSACK_IB	(.O(USR_CLOSE_ACK), .I(USR_CLOSE_ACK_IN));
	IBUF								USR_TXWE_IB		(.O(USR_TX_WE),		.I(USR_TX_WE_IN));
	IBUF								USR_RXRE_IB		(.O(USR_RX_RE),		.I(USR_RX_RE_IN));
*/
/*
	generate
		genvar USR_D_var ; 
		 for (USR_D_var=0;USR_D_var<8;USR_D_var=USR_D_var+1) begin : USR_D_LOOP
			IBUF								USR_TXD_IB	(.O(USR_TX_WD[USR_D_var]),	.I(USR_TX_WD_IN[USR_D_var]));
			OBUF	#(.DRIVE(4), .SLEW("FAST"))	USR_RXD_OB	(.O(USR_RX_RD_OUT[USR_D_var]),.I(USR_RX_RD[USR_D_var]));
		 end
	endgenerate
*/
/*
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	USR_ACTV_OB	(.O(USR_ACTIVE_OUT),	.I(USR_ACTIVE));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	USR_SPND_OB	(.O(USR_SUSPEND_OUT),	.I(USR_SUSPEND));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	USR_CLSR_OB	(.O(USR_CLOSE_REQ_OUT),	.I(USR_CLOSE_REQ));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	USR_TXFL_OB	(.O(USR_TX_AFULL_OUT),	.I(USR_TX_AFULL));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	USR_RXEP_OB	(.O(USR_RX_EMPTY_OUT),	.I(USR_RX_EMPTY));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	USR_RXRV_OB	(.O(USR_RX_RV_OUT),		.I(USR_RX_RV));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	REG_CLK_OB	(.O(REG_CLK_OUT),		.I(REG_CLK));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	REG_ACT_OB	(.O(REG_ACT_OUT),		.I(REG_ACT));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	REG_DO_OB	(.O(REG_DO_OUT),		.I(REG_DO));
	IBUF								REG_DI_IB	(.O(REG_DI),			.I(REG_DI_IN));
*/
	generate
		genvar JMPR_var ; 
		 for (JMPR_var=0;JMPR_var<4;JMPR_var=JMPR_var+1) begin : JMPR_LOOP
			IBUF								DIPSW_IB	(.O(DIPSW[JMPR_var]), .I(DIPSW_IN[JMPR_var]));
			PULLUP								JMPR_PU		(.O(DIPSW_IN[JMPR_var]));
		 end
	endgenerate

	IBUF								nFD_IB		(.O(FORCE_DEFAULTn),.I(nFD_IN));
	PULLUP								FD_PU		(.O(nFD_IN));

	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	PNLLED_G_OB	(.O(PNLLED_G_OUT), .I(PNLLED_G));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	PNLLED_Y_OB	(.O(PNLLED_Y_OUT), .I(PNLLED_Y));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	PNLLED_R_OB	(.O(PNLLED_R_OUT), .I(PNLLED_R));

	IBUF								SW_IB		(.O(PUSHSW),			.I(PUSHSW_IN));

	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	D1_LED_OB1	(.O(SOY2_D1_LED_OUT[0]), .I(SOY2_D1_LED[0]));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	D1_LED_OB2	(.O(SOY2_D1_LED_OUT[1]), .I(SOY2_D1_LED[1]));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	D1_LED_OB3	(.O(SOY2_D1_LED_OUT[2]), .I(SOY2_D1_LED[2]));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	D1_LED_OB4	(.O(SOY2_D1_LED_OUT[3]), .I(SOY2_D1_LED[3]));
	IBUF								D1_COMP_IB1	(.O(SOY2_D1_COMP[0]), .I(SOY2_D1_COMP_IN[0]));
	IBUF								D1_COMP_IB2	(.O(SOY2_D1_COMP[1]), .I(SOY2_D1_COMP_IN[1]));
	IBUF								D1_COMP_IB3	(.O(SOY2_D1_COMP[2]), .I(SOY2_D1_COMP_IN[2]));
	IBUF								D1_COMP_IB4	(.O(SOY2_D1_COMP[3]), .I(SOY2_D1_COMP_IN[3]));
	generate
		genvar Pmod_var ; 
		 for (Pmod_var=0;Pmod_var<8;Pmod_var=Pmod_var+1) begin : Pmod_LOOP
			OBUF	#(.DRIVE(4), .SLEW("SLOW"))	D1_Pmod_OB	(.O(SOY2_D1_Pmod_OUT[Pmod_var]), .I(SOY2_D1_Pmod[Pmod_var]));
		 end
	endgenerate

//------------------------------------------------------------------------------
//	Reconfig (with push switch)
//------------------------------------------------------------------------------
	always@(posedge SYSCLK200M)begin
		if(PUSHSW)begin
			SWTIME_1US[8:0]		<=  9'd198;
			SWTIME_100US[7:0]	<=  8'd99;
			SWTIME_100MS[10:0]	<= 11'd999;
			P0_SWPLS_100MS		<=  1'b0;
			P1_SWPLS_100MS		<=  1'b0;
		end else begin
			SWTIME_1US[8:0]		<= SWTIME_1US[8]?		 9'd198:	(SWTIME_1US[8:0]   - {8'd0, ~PUSHSW});
			SWTIME_100US[7:0]	<= SWTIME_100US[7]?		 8'd99:		(SWTIME_100US[7:0] - {7'd0,  SWTIME_1US[8]});
			SWTIME_100MS[10:0]	<= SWTIME_100MS[10]?	11'd999:	(SWTIME_100MS[10:0]- {10'd0, SWTIME_100US[7]});
			P0_SWPLS_100MS		<= SWTIME_100MS[10];
			P1_SWPLS_100MS		<= SWTIME_100MS[10] & ~P0_SWPLS_100MS;
		end
	end

	assign	REBOOT_REQ	= P1_SWPLS_100MS|BOOT_REQ;

	REBOOT	#(
		.CLOCK_FREQUENCY	(8'd200			)			// CLK_IN Frequency(MHz) > 2MHz
	)
	REBOOT	(
		.CLK_IN				(SYSCLK200M		),			// System Clock
		.CFG_CLK_IN			(CLK_100M		),			// Reconfig Clock =< 100MHz
		.RESET_IN			(SYS_RST		),			// Reset
		.REBOOT_IN			(REBOOT_REQ)					// Reboot Request
	);


//------------------------------------------------------------------------------
//	25MHz Generator(PHY_CLK)
//------------------------------------------------------------------------------
	initial		CNT25M[1:0]	= 2'b00;
	initial		RE_CLK25M	= 1'b0;
	initial		FE_CLK25M	= 1'b0;

	always@(posedge CLK_125M)begin
		CNT25M[0]	<= ~CNT25M[0] & ~(~CNT25M[1] & RE_CLK25M);
		CNT25M[1]	<= CNT25M[0]^CNT25M[1];
		RE_CLK25M	<= CNT25M[0]|CNT25M[1];
	end

	always@(negedge CLK_125M)begin
		FE_CLK25M	<= CNT25M[1];
	end

	ODDR	ETH_MACCLK_OR(.Q(ETH_PHY_CLK), .C(CLK_125M), .CE(1'b1), .D1(RE_CLK25M), .D2(FE_CLK25M), .R(1'b0), .S(1'b0));
	OBUF	#(.DRIVE(4), .SLEW("SLOW"))	PHY_25M_OB	(.O(ETH_PHY_CLK_OUT), .I(ETH_PHY_CLK));


//------------------------------------------------------------------------------
//	DEMO mode (Couter output)
//------------------------------------------------------------------------------
	assign	DEMO_ENB	= SiTCP_OPEN_ACK & ~SiTCP_TX_AFULL;

	always@ (posedge SYSCLK200M or posedge SiTCP_RST) begin
		if(SiTCP_RST)begin
			reg_SiTCP_TX_WR			<= 1'b0;
			DEMO_DATA[7:0]			<= 8'd0;
			reg_SiTCP_TX_DATA[7:0]	<= 8'd0;
		end else begin
			reg_SiTCP_TX_WR			<= DEMO_ENB;
			DEMO_DATA[7:0]			<= DEMO_DATA[7:0] + {7'd0, DEMO_ENB};
			reg_SiTCP_TX_DATA[7:0]	<= DEMO_DATA[7:0];
		end 
	end

	always@ (posedge SYSCLK200M)begin
		reg_SiTCP_CLOSE_ACK	<= SiTCP_CLOSE_REQ;
	end

	assign	SiTCP_TX_WR			= ~DIPSW[0]?	reg_SiTCP_TX_WR:		wi_SiTCP_TX_WR;
	assign	SiTCP_TX_DATA[7:0]	= ~DIPSW[0]?	reg_SiTCP_TX_DATA[7:0]:	wi_SiTCP_TX_DATA[7:0];
	assign	SiTCP_CLOSE_ACK		= ~DIPSW[0]?	reg_SiTCP_CLOSE_ACK:	wi_SiTCP_CLOSE_ACK;

//------------------------------------------------------------------------------
//	Timer
//------------------------------------------------------------------------------
	TIMER  #(
		.TIM_PERIOD				(8'd198)
	)
	TIMER	(
	// System
		.CLK					(SYSCLK200M				),	// in	: System clock
		.RST					(SYS_RST				),	// in	: System reset
	// Intrrupts
		.TIM_1US				(TIM_1US				),	// out	: 1 us interval
		.TIM_1MS				(TIM_1MS				),	// out	: 1 ms interval
		.TIM_1S					(TIM_1S					),	// out	: 1 s interval
		.TIM_1M					(TIM_1MIN				)	// out	: 1 min interval
	);

/*
//------------------------------------------------------------------------------
//	User I/F
//------------------------------------------------------------------------------

	SiTCP_EV_USRIF		SiTCP_EV_USRIF	(
	// System
		.CLK					(SYSCLK200M				),	// in	: System clock
		.RST					(SiTCP_RST				),	// in	: System reset
	// SiTCP IF
		.OPEN_ACK				(SiTCP_OPEN_ACK			),	// in	: Acknowledge for open (=Socket busy)
		.CLOSE_REQ				(SiTCP_CLOSE_REQ		),	// in	: Connection close
		.CLOSE_ACK				(wi_SiTCP_CLOSE_ACK		),	// out	: Acknowledge for close
		.RX_FIFO_FILL			(SiTCP_RX_FILL[15:0]	),	// out	: Fill level[15:0]
		.RX_FIFO_WR				(SiTCP_RX_WR			),	// in	: Write enable
		.RX_FIFO_DATA			(SiTCP_RX_DATA[7:0]		),	// in	: Write data[7:0]
		.TX_FIFO_AFULL			(SiTCP_TX_AFULL			),	// in	: Almost full flag
		.TX_FIFO_WR				(wi_SiTCP_TX_WR			),	// out	: Write enable
		.TX_FIFO_DATA			(wi_SiTCP_TX_DATA[7:0]	),	// out	: Write data[7:0]
	// User I/F
		.USR_CLK				(USR_CLK				),	// in	: Clock
		.USR_ACTIVE				(USR_ACTIVE				),	// out	: TCP established
		.USR_CLOSE_REQ			(USR_CLOSE_REQ			),	// out	: Request to close a TCP Connection
		.USR_CLOSE_ACK			(USR_CLOSE_ACK			),	// in	: Acknowledge for USR_CLOSE_REQ
		// TCP Tx
		.USR_TX_AFULL			(USR_TX_AFULL			),	// out	: Almost full flag of a TCP Tx FIFO
		.USR_TX_WE				(USR_TX_WE				),	// in	: TCP Tx Data write enable
		.USR_TX_WD				(USR_TX_WD[7:0]			),	// in	: TCP Tx Data[7:0]
		// TCP Rx
		.USR_RX_EMPTY			(USR_RX_EMPTY			),	// out	: Empty flag of a TCP RX flag
		.USR_RX_RE				(USR_RX_RE				),	// in	: TCP Rx Data Read enable
		.USR_RX_RV				(USR_RX_RV				),	// out	: TCP Rx data valid
		.USR_RX_RD				(USR_RX_RD[7:0]			)	// out	: TCP Rx data[7:0]
	);
*/

/*
	SIO_MASTER			SIO_MASTER	(
		// System
		.CLK					(SYSCLK200M				),	// in	: System clock
		.RST					(SiTCP_RST				),	// in	: System reset
		// Timer
		.TIM_1US				(TIM_1US				),	// in	: 1us interrupt
		.ADDR_WIDTH				(2'd3					),	// in	: Address width[1:0] (0: 8bit, 1:16bit, 2:24bit, 3:32bit)
		// SiTCP I/F
		.LOC_ACT				(LOC_ACT				),	// in	: Request
		.LOC_ADDR				(LOC_ADDR[31:0]			),	// in	: Address[31:0]
		.LOC_WD					(LOC_WD[7:0]			),	// in	: Data[7:0]
		.LOC_WE					(LOC_WE					),	// in	: Write enable
		.LOC_RE					(LOC_RE					),	// in	: Read enable
		.LOC_ACK				(SIO_LOC_ACK			),	// out	: Access acknowledge
		.LOC_RD					(SIO_LOC_RD[7:0]		),	// out	: Read data[7:0]
		// Serial I/F
		.SCK					(REG_CLK				),	// out	: Clock
		.SCS					(REG_ACT				),	// out	: Active
		.SI						(REG_DI					),	// in	: Data input
		.SO						(REG_DO					)	// out	: Data output
	);
*/


//------------------------------------------------------------------------------
//	SiTCP
//------------------------------------------------------------------------------

	WRAP_SiTCP_GMII_XC7S_32K	#(
		.TIM_PERIOD			(8'd200				)
	)
	SiTCP(
		.CLK				(SYSCLK200M   		),	// in	: System Clock >129MHz
		.RST				(SYS_RST			),	// in	: System reset
	// Configuration parameters
		.FORCE_DEFAULTn		(FORCE_DEFAULTn		),	// in	: Load default parameters
		.EXT_IP_ADDR		(32'd0				),	// in	: IP address[31:0]
		.EXT_TCP_PORT		(16'd0				),	// in	: TCP port #[15:0]
		.EXT_RBCP_PORT		(16'd0				),	// in	: RBCP port #[15:0]
		.PHY_ADDR			(5'b0_0000			),	// in	: PHY-device MIF address[4:0]
	// EEPROM
		.EEPROM_CS			(EEPROM_CS			),	// out	: Chip select
		.EEPROM_SK			(EEPROM_SK			),	// out	: Serial data clock
		.EEPROM_DI			(EEPROM_DI			),	// out	: Serial write data
		.EEPROM_DO			(EEPROM_DO			),	// in	: Serial read data
	// user data, intialial values are stored in the EEPROM, 0xFFFF_FC3C-3F
		.USR_REG_X3C		(					),	// out	: Stored at 0xFFFF_FF3C
		.USR_REG_X3D		(					),	// out	: Stored at 0xFFFF_FF3D
		.USR_REG_X3E		(					),	// out	: Stored at 0xFFFF_FF3E
		.USR_REG_X3F		(					),	// out	: Stored at 0xFFFF_FF3F
	// MII interface
		.GMII_RSTn			(GMII_RSTn			),	// out	: PHY reset Active low
		.GMII_1000M			(GMII_1000M			),	// in	: GMII mode (0:MII, 1:GMII)
		// TX
		.GMII_TX_CLK		(GMII_TX_CLK		),	// in	: Tx clock
		.GMII_TX_EN			(GMII_TX_EN			),	// out	: Tx enable
		.GMII_TXD			(GMII_TXD[7:0]		),	// out	: Tx data[7:0]
		.GMII_TX_ER			(GMII_TX_ER			),	// out	: TX error
	// RX
		.GMII_RX_CLK		(GMII_RX_CLK		),	// in	: Rx clock
		.GMII_RX_DV			(GMII_RX_DV			),	// in	: Rx data valid
		.GMII_RXD			(GMII_RXD[7:0]		),	// in	: Rx data[7:0]
		.GMII_RX_ER			(GMII_RX_ER			),	// in	: Rx error
		.GMII_CRS			(GMII_CRS			),	// in	: Carrier sense
		.GMII_COL			(GMII_COL			),	// in	: Collision detected
	// Management IF
		.GMII_MDC			(GMII_MDC			),	// out	: Clock for MDIO
		.GMII_MDIO_IN		(GMII_MDIO_IN		),	// in	: Data
		.GMII_MDIO_OUT		(GMII_MDIO_OUT		),	// out	: Data
		.GMII_MDIO_OE		(GMII_MDIO_OE		),	// out	: MDIO output enable
	// User I/F
		.SiTCP_RST			(SiTCP_RST			),	// out	: Reset for SiTCP and related circuits
		// TCP connection control
		.TCP_OPEN_REQ		(1'b0				),	// in	: Reserved input, shoud be 0
		.TCP_OPEN_ACK		(SiTCP_OPEN_ACK		),	// out	: Acknowledge for open (=Socket busy)
		.TCP_ERROR			(					),	// out	: TCP error, its active period is equal to MSL
		.TCP_CLOSE_REQ		(SiTCP_CLOSE_REQ	),	// out	: Connection close request
		.TCP_CLOSE_ACK		(SiTCP_CLOSE_ACK	),	// in		: Acknowledge for closing
		// FIFO I/F
		.TCP_RX_WC			(1'b1				),	// in	: Rx FIFO write count[15:0] (Unused bits should be set 1)
		.TCP_RX_WR			(SiTCP_RX_WR		),	// out	: Write enable
		.TCP_RX_DATA		(SiTCP_RX_DATA[7:0]	),	// out	: Write data[7:0]
		.TCP_TX_FULL		(SiTCP_TX_AFULL		),	// out	: Almost full flag
		.TCP_TX_WR			(SiTCP_TX_WR		),	// in	: Write enable
		.TCP_TX_DATA		(SiTCP_TX_DATA[7:0]	),	// in	: Write data[7:0]
		// RBCP
		.RBCP_ACT			(RBCP_ACT			),	// out	: RBCP active
		.RBCP_ADDR			(RBCP_ADDR[31:0]	),	// out	: Address[31:0]
		.RBCP_WD			(RBCP_WD[7:0]		),	// out	: Data[7:0]
		.RBCP_WE			(RBCP_WE			),	// out	: Write enable
		.RBCP_RE			(RBCP_RE			),	// out	: Read enable
		.RBCP_ACK			(RBCP_ACK			),	// in	: Access acknowledge
		.RBCP_RD			(RBCP_RD[7:0]		)	// in	: Read data[7:0]
	);

//	assign	LOC_ACK		= ~DIPSW[0]?	RBCP_DEMOACK:		SIO_LOC_ACK;
//	assign	LOC_RD[7:0]	= ~DIPSW[0]?	RBCP_DEMODATA[7:0]:	SIO_LOC_RD[7:0];

/*
//------------------------------------------------------------------------------
//	Version Register(RBCP)
//------------------------------------------------------------------------------
	always@ (posedge SYSCLK200M) begin
		DEC_ADDR[1]			<= (LOC_ADDR[31:28] == 4'h0);
		DEC_ADDR[0]			<= (LOC_ADDR[ 5: 2] == 4'h0);
		P0_RDT[7:0]			<= (
			((LOC_ADDR[1:0] == 2'b00)?	VERSION_REG[31:24]:	8'h00) |
			((LOC_ADDR[1:0] == 2'b01)?	VERSION_REG[23:16]:	8'h00) |
			((LOC_ADDR[1:0] == 2'b10)?	VERSION_REG[15: 8]:	8'h00) |
			((LOC_ADDR[1:0] == 2'b11)?	VERSION_REG[ 7: 0]:	8'h00)
		);
		RBCP_DEMODATA[7:0]	<= P0_RE?	P0_RDT[7:0]:	RBCP_DEMODATA[7:0];	// only in Demo Mode	#180926
		P0_RE				<=  LOC_RE;
		P0_WE				<=  LOC_WE;
		RBCP_DEMOACK		<= (&DEC_ADDR[1:0]) & (P0_RE | P0_WE);			// only in Demo Mode	#180926
	end
*/
//------------------------------------------------------------------------------
// RBCP Register
//------------------------------------------------------------------------------

	REG_CTL	#(
		.VERSION			(`FPGA_VER),
		.FPGA_NAME			(`FPGA_NAME)
	 )
	REG_CTL(
		// RBCP2SiTCP Interconnect
		.SYSCLK_IN			(SYSCLK200M),				// System Clock(200MHz)
		.RESET_IN			(SiTCP_RST),				// Register Reset(SiTCP_RST)
		.RBCP_ACT_IN		(RBCP_ACT),					// RBCP active
		.RBCP_ADDR_IN		(RBCP_ADDR[31:0]),			// Address[31:0]
		.RBCP_WD_IN			(RBCP_WD[7:0]),				// Data[7:0]
		.RBCP_WE_IN			(RBCP_WE),					// Write enable
		.RBCP_RE_IN			(RBCP_RE),					// Read enable
		.RBCP_ACK_OUT		(RBCP_ACK),					// Access acknowledge
		.RBCP_RD_OUT		(RBCP_RD[7:0]),				// Read data[7:0]
		.BOOT_REQ_OUT		(BOOT_REQ),					// Boot Request
		
		// Registers
//		.REG_D1_LED			(SOY2_D1_LED[3:0]),			//out 	SOY2_D1_LED[3:0]
		.SOY2_D1_COMP_IN	(SOY2_D1_COMP[3:0]),		//in	SOY2_D1 Comparator[3:0]
//		.REG_D1_Pmod		(SOY2_D1_Pmod[7:0]),		//out	SOY2_D1 Pmod pin[7:0]
		.REG_D1_DAC_A		(REG_D1_DAC_A[11:0]),		//out	SOY2_D1 DAC_A[11:0]
		.REG_D1_DAC_B		(REG_D1_DAC_B[11:0]),		//out	SOY2_D1 DAC_B[11:0]
		.REG_D1_DAC_C		(REG_D1_DAC_C[11:0]),		//out	SOY2_D1 DAC_C[11:0]
		.REG_D1_DAC_D		(REG_D1_DAC_D[11:0]),		//out	SOY2_D1 DAC_D[11:0]
		.SOY2_D1_ADC_A_IN	(SOY2_D1_ADC_A[11:0]),		//in	SOY2_D1 ADC_A[11:0]
		.SOY2_D1_ADC_B_IN	(SOY2_D1_ADC_B[11:0]),		//in	SOY2_D1 ADC_B[11:0]
		
		// Registers' Handshaking
		.P1_WEN				(P1_WEN[11:0]),				//out	RBCP wrote valid[11:0]
		.REG_WDT			(REG_WDT[7:0])				//out	RBCP wrote data[7:0]
	);



//------------------------------------------------------------------------------
//	Ethernet Active LED
//------------------------------------------------------------------------------

    //count 128ms for LED
	always@ (posedge SYSCLK200M or posedge SiTCP_RST) begin
		if(SiTCP_RST)begin
			CNTLED[7:0]	<= 8'd0;
		end else begin
			if (TIM_1MS) begin
				CNTLED[7:0]	<= CNTLED[7:0] + 8'd1;
			end
		end
	end

	assign	PNLLED_G	=  1'b1;
//	assign	PNLLED_Y	= ~DIPSW[0]?	SiTCP_OPEN_ACK:	USR_ACTIVE;		// TCP Session(Demo mode) or USR_ACTIVE
	assign	PNLLED_Y	=  SiTCP_OPEN_ACK;								// TCP Session(Demo mode)
	assign	PNLLED_R	=  SiTCP_RST | (~FORCE_DEFAULTn & CNTLED[7]);	// During SiTCP_RST or Force Default

//------------------------------------------------------------------------------
//	SOY2_D1
//------------------------------------------------------------------------------
	// comparator to LED
	assign SOY2_D1_LED[3:0] = SOY2_D1_COMP[3:0]; 
	
	//count for PMOD
	reg	[7:0]	SOY_D1_CNTLED;
	always@ (posedge SYSCLK200M or posedge SiTCP_RST) begin
		if(SiTCP_RST)begin
			SOY_D1_CNTLED[7:0]	<= 8'd0;
		end else begin
			if (TIM_1MS) begin
				SOY_D1_CNTLED[7:0]	<= SOY_D1_CNTLED[7:0] + 8'd1;
			end else if(SOY_D1_CNTLED[4]) begin
				SOY_D1_CNTLED[7:0]	<= 8'd0;
			end
		end
	end
	
//	always@ (posedge SYSCLK200M or posedge SiTCP_RST) begin
//		if(SiTCP_RST)begin
//			SOY2_D1_Pmod[7:0]	<= 8'd0;
//		end else begin
//			if (SOY_D1_CNTLED[4]) begin
//				SOY2_D1_Pmod[7:0]	<= SOY2_D1_Pmod[7:0] + 8'd1;
//			end
//		end
//	end


	always@ (posedge SYSCLK200M or posedge SiTCP_RST) begin
		if(SiTCP_RST)begin
			SOY2_D1_Pmod[7:0]	<= 8'd0;
		end else begin
			if (SOY_D1_CNTLED[4]) begin
				SOY2_D1_Pmod[7:0]	<= {~(SOY2_D1_Pmod[3:0]	+	4'd1),SOY2_D1_Pmod[3:0]	+	4'd1};
			end
		end
	end

//------------------------------------------------------------------------------
//	SOY2_D1 DAC
//------------------------------------------------------------------------------
	DAC_CTL 	#(
		.FREQUENCY		(8'd200)					// CLK_IN Frequency  > 10MHz
	)
	DAC_CTL	(
		.CLK_IN			(SYSCLK200M),			// System Clock
		.RESET_IN		(SYS_RST),				// Reset
		// Device Interface
		.IIC_SCL_OUT	(IIC_SCL_OUT),			// IIC Clock
		.IIC_SDA_IO		(IIC_SDA_IO),			// IIC Data
		
		.DAC_WDT		(DAC_WDT[11:0]),		// DAC write Data
		.DAC_CH			(DAC_CH[1:0]),			// DAC Channel Select (00:1ch, 01:2ch, 10:3ch, 11:4ch)
		.DAC_REQ		(DAC_REQ),				// DAC Request
		
		.DAC_RAK_OUT	(DAC_RAK)				// DAC Request Acknowledge
	);
	wire	[11:0]	REG_D1_DAC_A;				// SOY2_D1 DAC_A[11:0]
	wire	[11:0]	REG_D1_DAC_B;				// SOY2_D1 DAC_B[11:0]
	wire	[11:0]	REG_D1_DAC_C;				// SOY2_D1 DAC_C[11:0]
	wire	[11:0]	REG_D1_DAC_D;				// SOY2_D1 DAC_D[11:0]
	
	reg	    [11:0]	DAC_WDT;
	reg		[1:0]	DAC_CH;						// DAC Channel Select (00:1ch, 01:2ch, 10:3ch, 11:4ch)
	reg				DAC_REQ;					// DAC Request
	wire			DAC_RAK;					// DAC Request Acknowledge

	always@ (posedge SYSCLK200M) begin
		if(SYS_RST) begin
			IIC_RDY					<= 1'b0;
			IIC_nLDAC				<= 1'b0;
			DAC_REQ	<=	1'b0;
			DAC_CH	<=	2'b11;
			DAC_WDT  <=  12'd0;
		end else begin
			DAC_REQ	<=	(~DAC_REQ	&	(TIM_1MS))	|	(DAC_REQ	&	(~DAC_RAK));	//Send request to DAC
			DAC_CH	<=	(DAC_CH[1:0]	+	TIM_1MS);	//Channel Select
			DAC_WDT	<=	(
						(DAC_CH	==	2'b00)?	REG_D1_DAC_A[11:0]:	12'd0|					//Data Select Register A
						(DAC_CH	==	2'b01)?	REG_D1_DAC_B[11:0]:	12'd0|					//Data Select Register B
						(DAC_CH	==	2'b10)?	REG_D1_DAC_C[11:0]:	12'd0|					//Data Select Register C
						(DAC_CH	==	2'b11)?	REG_D1_DAC_D[11:0]:	12'd0					//Data Select Register D
						);
		end
	end

//------------------------------------------------------------------------------
//	SOY2_D1 ADC ADS7253
//------------------------------------------------------------------------------
	// SPI INTERCONNECT
	//(* mark_debug = "true" *)	wire				SPI_REQ_IN;				// SPI Request	
	wire				SPI_RAK_OUT;			// SPI Request Acknowledge
	wire		[ 11:0]	SOY2_D1_ADC_A;			// SPI Read Data[11:0]
	wire		[ 11:0]	SOY2_D1_ADC_B;			// SPI Read Data[11:0]
	wire				SPI_RVL_OUT;			// SPI Read Data Valid

	
	// ADC SPI Control
	ADC_CTL 	#(
		.FREQUENCY		(8'd200),					// CLK_IN Frequency  > 10MHz
		.DRIVE			(4),						// Output Buffer Strength
		.IOSTANDARD		("DEFAULT"),				// I/O Standard
		.SLEW			("SLOW")					// Outputbufer Slew rate
	)
	ADC_CTL		(
		// SPI INTERCONNECT
		.CLK_IN			(SYSCLK200M),				// System Clock
		.RESET_IN		(SYS_RST),					// Reset
		
		.SPI_REQ_IN		(TIM_1MS),					// SPI Request
		.SPI_CFG_REG_IN	(16'b1000_0010_0100_0000),	// SPI Configuration Register Write Data[15:0] default:0b1000_0010_0100_0000		
		.SPI_RAK_OUT	(SPI_RAK_OUT),				// SPI Request Acknowledge
		.SPI_RDT_A_OUT	(SOY2_D1_ADC_A[11:0]),		// SPI Read Data[11:0]
		.SPI_RDT_B_OUT	(SOY2_D1_ADC_B[11:0]),		// SPI Read Data[11:0]
		.SPI_RVL_OUT	(SPI_RVL_OUT),				// SPI Read Data Valid

		// SPI INTERFACE
		.SPI_CLK_OUT	(SPI_CLK),					//
		.SPI_CSn_OUT	(SPI_CSn),					//	CHIP SELECT (active low)
		.SPI_MOSI_OUT	(SPI_MOSI),					//
		.SPI_MISO_A		(SPI_MISO_A),				//
		.SPI_MISO_B		(SPI_MISO_B)				//
	);

endmodule

`default_nettype wire
